@@ -15,6 +15,7 @@ from . import roles
 from . import schema
 from . import sqltypes
 from . import util as sqlutil
+from .base import _entity_namespace
 from .base import ColumnCollection
 from .base import Executable
 from .base import Generative
@@ -618,6 +619,16 @@ class FunctionElement(Executable, ColumnElement, FromClause, Generative):
         else:
             return super(FunctionElement, self).self_group(against=against)
 
+    @property
+    def entity_namespace(self):
+        """overrides FromClause.entity_namespace as functions are generally
+        column expressions and not FromClauses.
+
+        """
+        # ideally functions would not be fromclauses but we failed to make
+        # this adjustment in 1.4
+        return _entity_namespace(self.clause_expr)
+
 
 class FunctionAsBinary(BinaryExpression):
     _traverse_internals = [
@@ -790,7 +801,7 @@ class _FunctionGenerator(object):
                 return func(*c, **o)
 
         return Function(
-            self.__names[-1], packagenames=self.__names[0:-1], *c, **o
+            self.__names[-1], packagenames=tuple(self.__names[0:-1]), *c, **o
         )
 
 
@@ -1064,6 +1075,9 @@ class next_value(GenericFunction):
         ), "next_value() accepts a Sequence object as input."
         self._bind = self._get_bind(kw)
         self.sequence = seq
+        self.type = sqltypes.to_instance(
+            seq.data_type or getattr(self, "type", None)
+        )
 
     def compare(self, other, **kw):
         return (
